/*!
FullCalendar Resource Day Grid Plugin v7.0.0-beta.0
Docs & License: https://fullcalendar.io/docs/resource-daygrid-view
(c) 2024 Adam Shaw
*/
FullCalendar.ResourceDayGrid = (function (exports, core, premiumCommonPlugin, resourcePlugin, dayGridPlugin, internal$2, preact, internal$3, internal$1) {
    'use strict';

    function _interopDefault (e) { return e && e.__esModule ? e : { 'default': e }; }

    var premiumCommonPlugin__default = /*#__PURE__*/_interopDefault(premiumCommonPlugin);
    var resourcePlugin__default = /*#__PURE__*/_interopDefault(resourcePlugin);
    var dayGridPlugin__default = /*#__PURE__*/_interopDefault(dayGridPlugin);

    class ResourceDayTableJoiner extends internal$1.VResourceJoiner {
        transformSeg(seg, resourceDayTableModel, resourceI) {
            let colRanges = resourceDayTableModel.computeColRanges(seg.firstCol, seg.lastCol, resourceI);
            return colRanges.map((colRange) => (Object.assign(Object.assign(Object.assign({}, seg), colRange), { isStart: seg.isStart && colRange.isStart, isEnd: seg.isEnd && colRange.isEnd })));
        }
    }

    function buildResourceHeaderTiers(resources, dates, datesAboveResources, datesRepDistinctDays, context) {
        if (!resources.length) {
            return [
                datesRepDistinctDays
                    ? dates.map((date) => ({ date, colSpan: 1 }))
                    : dates.map((date) => ({ dow: date.getUTCDay(), colSpan: 1 }))
            ];
        }
        if (dates.length === 1) {
            return [
                resources.map((resource) => ({ type: 'resource', colSpan: 1, resource, date: dates[0] }))
            ];
        }
        if (datesAboveResources) {
            return [
                datesRepDistinctDays
                    ? dates.map((date) => ({ date, colSpan: resources.length }))
                    : dates.map((date) => ({ dow: date.getUTCDay(), colSpan: resources.length })),
                [].concat(...dates.map((date) => (resources.map((resource) => ({ resource, date: datesRepDistinctDays ? date : undefined, colSpan: 1 })))))
            ];
        }
        return [
            resources.map((resource) => ({ resource, date: dates[0], colSpan: dates.length })),
            datesRepDistinctDays
                ? [].concat(...resources.map((resource) => (dates.map((date) => ({
                    date,
                    colSpan: 1,
                    extraRenderProps: { resource: new resourcePlugin.ResourceApi(context, resource) },
                    extraDataAttrs: { 'data-resource-id': resource.id },
                })))))
                : [].concat(...resources.map((resource) => (dates.map((date) => ({
                    dow: date.getUTCDay(),
                    colSpan: 1,
                    extraRenderProps: { resource: new resourcePlugin.ResourceApi(context, resource) },
                    extraDataAttrs: { 'data-resource-id': resource.id },
                })))))
        ];
    }

    class ResourceHeaderCell extends internal$2.BaseComponent {
        constructor() {
            super(...arguments);
            // ref
            this.innerElRef = preact.createRef();
        }
        render() {
            let { props } = this;
            return (preact.createElement(internal$1.ResourceLabelContainer, { elTag: "div", elClasses: [
                    'fc-resource',
                    'fc-header-cell',
                    'fc-cell',
                    props.colWidth != null ? '' : 'fc-liquid',
                    'fc-flex-column',
                    'fc-align-center',
                ], elStyle: {
                    width: props.colWidth != null // TODO: DRY
                        ? props.colWidth * (props.colSpan || 1)
                        : undefined,
                }, resource: props.resource, date: props.date }, (InnerContent) => (preact.createElement("div", { ref: this.innerElRef, className: [
                    'fc-flex-column',
                    props.isSticky ? 'fc-sticky-x' : '',
                ].join(' ') },
                preact.createElement(InnerContent, { elTag: "span", elClasses: [
                        'fc-cell-inner',
                        'fc-padding-sm',
                    ] })))));
        }
        componentDidMount() {
            const innerEl = this.innerElRef.current; // TODO: make dynamic with useEffect
            // TODO: only attach this if refs props present
            this.disconectInnerHeight = internal$2.watchHeight(innerEl, (height) => {
                internal$2.setRef(this.props.innerHeightRef, height);
            });
        }
        componentWillUnmount() {
            this.disconectInnerHeight();
        }
    }

    class ResourceDayGridView extends internal$2.DateComponent {
        constructor() {
            super(...arguments);
            // memo
            this.flattenResources = internal$2.memoize(internal$1.flattenResources);
            this.buildResourceDayTableModel = internal$2.memoize(buildResourceDayTableModel);
            this.createDayHeaderFormatter = internal$2.memoize(internal$3.createDayHeaderFormatter);
            this.splitter = new internal$1.VResourceSplitter();
            this.slicers = {};
            this.joiner = new ResourceDayTableJoiner();
            this.isHitComboAllowed = (hit0, hit1) => {
                let allowAcrossResources = this.resourceDayTableModel.dayTableModel.colCnt === 1;
                return allowAcrossResources || hit0.dateSpan.resourceId === hit1.dateSpan.resourceId;
            };
        }
        render() {
            let { props, context } = this;
            let { options } = context;
            let resourceOrderSpecs = options.resourceOrder || internal$1.DEFAULT_RESOURCE_ORDER;
            let resources = this.flattenResources(props.resourceStore, resourceOrderSpecs);
            let resourceDayTableModel = this.resourceDayTableModel = this.buildResourceDayTableModel(props.dateProfile, context.dateProfileGenerator, resources, options.datesAboveResources, context);
            let splitProps = this.splitter.splitProps({
                resourceDayTableModel,
                businessHours: props.businessHours,
                dateSelection: props.dateSelection,
                eventStore: props.eventStore,
                eventUiBases: props.eventUiBases,
                eventSelection: props.eventSelection,
                eventDrag: props.eventDrag,
                eventResize: props.eventResize,
            });
            this.slicers = internal$2.mapHash(splitProps, (split, resourceId) => this.slicers[resourceId] || new internal$3.DayTableSlicer());
            let slicedProps = internal$2.mapHash(this.slicers, (slicer, resourceId) => slicer.sliceProps(splitProps[resourceId], props.dateProfile, options.nextDayThreshold, context, resourceDayTableModel.dayTableModel));
            let joinedSlicedProps = this.joiner.joinProps(slicedProps, resourceDayTableModel);
            let datesRepDistinctDays = resourceDayTableModel.dayTableModel.rowCnt === 1;
            let headerTiers = buildResourceHeaderTiers(// TODO: memoize???
            resources, resourceDayTableModel.dayTableModel.headerDates, options.datesAboveResources, datesRepDistinctDays, context);
            let dayHeaderFormat = this.createDayHeaderFormatter(context.options.dayHeaderFormat, datesRepDistinctDays, resourceDayTableModel.colCnt);
            return (preact.createElement(internal$2.NowTimer, { unit: "day" }, (nowDate, todayRange) => (preact.createElement(internal$3.DayGridLayout, { dateProfile: props.dateProfile, todayRange: todayRange, cellRows: resourceDayTableModel.cells, forPrint: props.forPrint, isHitComboAllowed: this.isHitComboAllowed, className: 'fc-resource-daygrid-view', 
                // header content
                // TODO: DRY with ResourceDayGridView/ResourceTimeGridView
                headerTiers: headerTiers, renderHeaderContent: (model, tierNum, innerHeightRef, colWidth) => {
                    if (model.resource) {
                        return (preact.createElement(ResourceHeaderCell, Object.assign({}, model, { colSpan: model.colSpan, colWidth: colWidth, isSticky: tierNum < headerTiers.length - 1 })));
                    }
                    else if (model.date) {
                        return (preact.createElement(internal$3.DateHeaderCell, Object.assign({}, model, { navLink: resourceDayTableModel.dayTableModel.colCnt > 1, dateProfile: props.dateProfile, todayRange: todayRange, dayHeaderFormat: dayHeaderFormat, colSpan: model.colSpan, colWidth: colWidth })));
                    }
                    else {
                        preact.createElement(internal$3.DayOfWeekHeaderCell, Object.assign({}, model, { dayHeaderFormat: dayHeaderFormat, colSpan: model.colSpan, colWidth: colWidth }));
                    }
                }, getHeaderModelKey: (model) => (model.resource
                    ? model.resource.id
                    : model.date
                        ? model.date.toISOString()
                        : model.dow), 
                // body content
                fgEventSegs: joinedSlicedProps.fgEventSegs, bgEventSegs: joinedSlicedProps.bgEventSegs, businessHourSegs: joinedSlicedProps.businessHourSegs, dateSelectionSegs: joinedSlicedProps.dateSelectionSegs, eventDrag: joinedSlicedProps.eventDrag, eventResize: joinedSlicedProps.eventResize, eventSelection: joinedSlicedProps.eventSelection }))));
        }
    }
    function buildResourceDayTableModel(dateProfile, dateProfileGenerator, resources, datesAboveResources, context) {
        let dayTable = internal$3.buildDayTableModel(dateProfile, dateProfileGenerator);
        if (!resources.length) {
            return new internal$1.ResourcelessDayTableModel(dayTable);
        }
        return datesAboveResources ?
            new internal$1.DayResourceTableModel(dayTable, resources, context) :
            new internal$1.ResourceDayTableModel(dayTable, resources, context);
    }

    var plugin = core.createPlugin({
        name: '@fullcalendar/resource-daygrid',
        premiumReleaseDate: '2024-10-01',
        deps: [
            premiumCommonPlugin__default["default"],
            resourcePlugin__default["default"],
            dayGridPlugin__default["default"],
        ],
        initialView: 'resourceDayGridDay',
        views: {
            resourceDayGrid: {
                type: 'dayGrid',
                component: ResourceDayGridView,
                needsResourceData: true,
            },
            resourceDayGridDay: {
                type: 'resourceDayGrid',
                duration: { days: 1 },
            },
            resourceDayGridWeek: {
                type: 'resourceDayGrid',
                duration: { weeks: 1 },
            },
            resourceDayGridMonth: {
                type: 'resourceDayGrid',
                duration: { months: 1 },
                fixedWeekCount: true,
            },
        },
    });

    var internal = {
        __proto__: null,
        ResourceDayTableJoiner: ResourceDayTableJoiner,
        ResourceDayGridView: ResourceDayGridView,
        ResourceHeaderCell: ResourceHeaderCell,
        buildResourceHeaderTiers: buildResourceHeaderTiers
    };

    core.globalPlugins.push(plugin);

    exports.Internal = internal;
    exports["default"] = plugin;

    Object.defineProperty(exports, '__esModule', { value: true });

    return exports;

})({}, FullCalendar, FullCalendar.PremiumCommon, FullCalendar.Resource, FullCalendar.DayGrid, FullCalendar.Internal, FullCalendar.Preact, FullCalendar.DayGrid.Internal, FullCalendar.Resource.Internal);
