import { VResourceJoiner, ResourceLabelContainer, flattenResources, VResourceSplitter, DEFAULT_RESOURCE_ORDER, ResourcelessDayTableModel, DayResourceTableModel, ResourceDayTableModel } from '@fullcalendar/resource/internal.js';
import { BaseComponent, watchHeight, setRef, DateComponent, memoize, mapHash, NowTimer } from '@fullcalendar/core/internal.js';
import { createRef, createElement } from '@fullcalendar/core/preact.js';
import { createDayHeaderFormatter, DayTableSlicer, DayGridLayout, DateHeaderCell, DayOfWeekHeaderCell, buildDayTableModel } from '@fullcalendar/daygrid/internal.js';
import { ResourceApi } from '@fullcalendar/resource/index.js';

class ResourceDayTableJoiner extends VResourceJoiner {
    transformSeg(seg, resourceDayTableModel, resourceI) {
        let colRanges = resourceDayTableModel.computeColRanges(seg.firstCol, seg.lastCol, resourceI);
        return colRanges.map((colRange) => (Object.assign(Object.assign(Object.assign({}, seg), colRange), { isStart: seg.isStart && colRange.isStart, isEnd: seg.isEnd && colRange.isEnd })));
    }
}

function buildResourceHeaderTiers(resources, dates, datesAboveResources, datesRepDistinctDays, context) {
    if (!resources.length) {
        return [
            datesRepDistinctDays
                ? dates.map((date) => ({ date, colSpan: 1 }))
                : dates.map((date) => ({ dow: date.getUTCDay(), colSpan: 1 }))
        ];
    }
    if (dates.length === 1) {
        return [
            resources.map((resource) => ({ type: 'resource', colSpan: 1, resource, date: dates[0] }))
        ];
    }
    if (datesAboveResources) {
        return [
            datesRepDistinctDays
                ? dates.map((date) => ({ date, colSpan: resources.length }))
                : dates.map((date) => ({ dow: date.getUTCDay(), colSpan: resources.length })),
            [].concat(...dates.map((date) => (resources.map((resource) => ({ resource, date: datesRepDistinctDays ? date : undefined, colSpan: 1 })))))
        ];
    }
    return [
        resources.map((resource) => ({ resource, date: dates[0], colSpan: dates.length })),
        datesRepDistinctDays
            ? [].concat(...resources.map((resource) => (dates.map((date) => ({
                date,
                colSpan: 1,
                extraRenderProps: { resource: new ResourceApi(context, resource) },
                extraDataAttrs: { 'data-resource-id': resource.id },
            })))))
            : [].concat(...resources.map((resource) => (dates.map((date) => ({
                dow: date.getUTCDay(),
                colSpan: 1,
                extraRenderProps: { resource: new ResourceApi(context, resource) },
                extraDataAttrs: { 'data-resource-id': resource.id },
            })))))
    ];
}

class ResourceHeaderCell extends BaseComponent {
    constructor() {
        super(...arguments);
        // ref
        this.innerElRef = createRef();
    }
    render() {
        let { props } = this;
        return (createElement(ResourceLabelContainer, { elTag: "div", elClasses: [
                'fc-resource',
                'fc-header-cell',
                'fc-cell',
                props.colWidth != null ? '' : 'fc-liquid',
                'fc-flex-column',
                'fc-align-center',
            ], elStyle: {
                width: props.colWidth != null // TODO: DRY
                    ? props.colWidth * (props.colSpan || 1)
                    : undefined,
            }, resource: props.resource, date: props.date }, (InnerContent) => (createElement("div", { ref: this.innerElRef, className: [
                'fc-flex-column',
                props.isSticky ? 'fc-sticky-x' : '',
            ].join(' ') },
            createElement(InnerContent, { elTag: "span", elClasses: [
                    'fc-cell-inner',
                    'fc-padding-sm',
                ] })))));
    }
    componentDidMount() {
        const innerEl = this.innerElRef.current; // TODO: make dynamic with useEffect
        // TODO: only attach this if refs props present
        this.disconectInnerHeight = watchHeight(innerEl, (height) => {
            setRef(this.props.innerHeightRef, height);
        });
    }
    componentWillUnmount() {
        this.disconectInnerHeight();
    }
}

class ResourceDayGridView extends DateComponent {
    constructor() {
        super(...arguments);
        // memo
        this.flattenResources = memoize(flattenResources);
        this.buildResourceDayTableModel = memoize(buildResourceDayTableModel);
        this.createDayHeaderFormatter = memoize(createDayHeaderFormatter);
        this.splitter = new VResourceSplitter();
        this.slicers = {};
        this.joiner = new ResourceDayTableJoiner();
        this.isHitComboAllowed = (hit0, hit1) => {
            let allowAcrossResources = this.resourceDayTableModel.dayTableModel.colCnt === 1;
            return allowAcrossResources || hit0.dateSpan.resourceId === hit1.dateSpan.resourceId;
        };
    }
    render() {
        let { props, context } = this;
        let { options } = context;
        let resourceOrderSpecs = options.resourceOrder || DEFAULT_RESOURCE_ORDER;
        let resources = this.flattenResources(props.resourceStore, resourceOrderSpecs);
        let resourceDayTableModel = this.resourceDayTableModel = this.buildResourceDayTableModel(props.dateProfile, context.dateProfileGenerator, resources, options.datesAboveResources, context);
        let splitProps = this.splitter.splitProps({
            resourceDayTableModel,
            businessHours: props.businessHours,
            dateSelection: props.dateSelection,
            eventStore: props.eventStore,
            eventUiBases: props.eventUiBases,
            eventSelection: props.eventSelection,
            eventDrag: props.eventDrag,
            eventResize: props.eventResize,
        });
        this.slicers = mapHash(splitProps, (split, resourceId) => this.slicers[resourceId] || new DayTableSlicer());
        let slicedProps = mapHash(this.slicers, (slicer, resourceId) => slicer.sliceProps(splitProps[resourceId], props.dateProfile, options.nextDayThreshold, context, resourceDayTableModel.dayTableModel));
        let joinedSlicedProps = this.joiner.joinProps(slicedProps, resourceDayTableModel);
        let datesRepDistinctDays = resourceDayTableModel.dayTableModel.rowCnt === 1;
        let headerTiers = buildResourceHeaderTiers(// TODO: memoize???
        resources, resourceDayTableModel.dayTableModel.headerDates, options.datesAboveResources, datesRepDistinctDays, context);
        let dayHeaderFormat = this.createDayHeaderFormatter(context.options.dayHeaderFormat, datesRepDistinctDays, resourceDayTableModel.colCnt);
        return (createElement(NowTimer, { unit: "day" }, (nowDate, todayRange) => (createElement(DayGridLayout, { dateProfile: props.dateProfile, todayRange: todayRange, cellRows: resourceDayTableModel.cells, forPrint: props.forPrint, isHitComboAllowed: this.isHitComboAllowed, className: 'fc-resource-daygrid-view', 
            // header content
            // TODO: DRY with ResourceDayGridView/ResourceTimeGridView
            headerTiers: headerTiers, renderHeaderContent: (model, tierNum, innerHeightRef, colWidth) => {
                if (model.resource) {
                    return (createElement(ResourceHeaderCell, Object.assign({}, model, { colSpan: model.colSpan, colWidth: colWidth, isSticky: tierNum < headerTiers.length - 1 })));
                }
                else if (model.date) {
                    return (createElement(DateHeaderCell, Object.assign({}, model, { navLink: resourceDayTableModel.dayTableModel.colCnt > 1, dateProfile: props.dateProfile, todayRange: todayRange, dayHeaderFormat: dayHeaderFormat, colSpan: model.colSpan, colWidth: colWidth })));
                }
                else {
                    createElement(DayOfWeekHeaderCell, Object.assign({}, model, { dayHeaderFormat: dayHeaderFormat, colSpan: model.colSpan, colWidth: colWidth }));
                }
            }, getHeaderModelKey: (model) => (model.resource
                ? model.resource.id
                : model.date
                    ? model.date.toISOString()
                    : model.dow), 
            // body content
            fgEventSegs: joinedSlicedProps.fgEventSegs, bgEventSegs: joinedSlicedProps.bgEventSegs, businessHourSegs: joinedSlicedProps.businessHourSegs, dateSelectionSegs: joinedSlicedProps.dateSelectionSegs, eventDrag: joinedSlicedProps.eventDrag, eventResize: joinedSlicedProps.eventResize, eventSelection: joinedSlicedProps.eventSelection }))));
    }
}
function buildResourceDayTableModel(dateProfile, dateProfileGenerator, resources, datesAboveResources, context) {
    let dayTable = buildDayTableModel(dateProfile, dateProfileGenerator);
    if (!resources.length) {
        return new ResourcelessDayTableModel(dayTable);
    }
    return datesAboveResources ?
        new DayResourceTableModel(dayTable, resources, context) :
        new ResourceDayTableModel(dayTable, resources, context);
}

export { ResourceDayGridView, ResourceDayTableJoiner, ResourceHeaderCell, buildResourceHeaderTiers };
